@@ -1,13 +1,15 @@
-from . import input  # make PyCharm understand `pywebio.input.xxx` expression
+from . import input  # enable `pywebio.input.xxx` expression without `import pywebio.input`
 from . import output
 from . import platform
 from . import session
+from . import pin
 
 from .__version__ import __author__, __author_email__, __license__
 from .__version__ import __description__, __url__, __version__
 from .exceptions import SessionException, SessionClosedException, SessionNotFoundException
 from .platform import start_server
 from .platform.bokeh import try_install_bokeh_hook
+from .platform.page import config
 from .utils import STATIC_PATH
 
 try_install_bokeh_hook()
