@@ -7,7 +7,7 @@ import tornado.web
 
 from . import page
 from ..session import Session
-from .httpbased import HttpContext, HttpHandler
+from .adaptor.http import HttpContext, HttpHandler
 from .tornado import set_ioloop, _setup_server, open_webbrowser_on_server_started
 from .utils import cdn_validation, print_listen_address
 from ..utils import parse_file_size
@@ -159,6 +159,6 @@ def start_server(applications, port=8080, host='',
 
     print_listen_address(host, port)
     if auto_open_webbrowser:
-        tornado.ioloop.IOLoop.current().spawn_callback(open_webbrowser_on_server_started, host or 'localhost', port)
+        tornado.ioloop.IOLoop.current().spawn_callback(open_webbrowser_on_server_started, host or '127.0.0.1', port)
 
     tornado.ioloop.IOLoop.current().start()
